/**
 * Copyright (c) 2016-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.ant.task.monit.assertion.flowtree;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.tools.ant.Project;
import org.ow2.petals.ant.task.monit.AbstractMonitFlowTreesAntTaskRequiringRefId;
import org.ow2.petals.ant.task.monit.FlowTreeNode;
import org.ow2.petals.ant.task.monit.assertion.exception.BuildAssertParamInvalidValueException;
import org.ow2.petals.ant.task.monit.assertion.exception.BuildAssertParamMissingException;
import org.ow2.petals.ant.task.monit.assertion.flowtree.exception.BuildAssertFlowTreeNodeNotFoundException;
import org.ow2.petals.ant.task.monit.assertion.flowtreenode.AssertFlowTreeNode;
import org.ow2.petals.ant.task.monit.assertion.flowtreenode.FlowTreeNodeAssertion;
import org.ow2.petals.ant.task.monit.assertion.flowtreenode.exception.BuildFlowInstanceIdInvalidException;

/**
 * A Ant task to create asserts on a flow tree build from MONIT traces.
 * 
 * @author Christophe DENEUX - Linagora
 *
 */
public class AssertFlowTreeTask extends AbstractMonitFlowTreesAntTaskRequiringRefId {

    /**
     * The flow instance identifier of the flow tree root on which assertions must be done
     */
    private String flowInstanceId;

    /**
     * Assertions to execute on flow instances of the selected flow tree
     */
    private final List<AssertFlowTreeNode> assertionsOnFlowTreeNode = new ArrayList<>();

    /**
     * @param flowInstanceId
     */
    public void setFlowInstanceId(final String flowInstanceId) {
        this.flowInstanceId = flowInstanceId;
    }

    public void addAssertFlowTreeNode(final AssertFlowTreeNode assertionsOnAFlowInstance) {
        this.assertionsOnFlowTreeNode.add(assertionsOnAFlowInstance);
    }

    @Override
    public final void doTaskWithFlowTrees(final Collection<FlowTreeNode> flowTrees) throws Exception {

        boolean flowTreeNodeFound = false;
        for (final FlowTreeNode flowTree : flowTrees) {
            final FlowTreeNode selectedFlowTreeNode = flowTree.getFlowTreeNode(this.flowInstanceId);
            if (selectedFlowTreeNode != null) {
                // It's the flow tree node associated to the expected flow instance
                flowTreeNodeFound = true;

                this.doAssertions(selectedFlowTreeNode);

                break;
            }
        }

        if (!flowTreeNodeFound) {
            throw new BuildFlowInstanceIdInvalidException(this.getLocation());
        }

    }

    /**
     * Execute assertions on the given root flow tree node.
     * 
     * @param rootFlowTreeNode
     *            The root flow tree node on which assertions must be done. Not {@code null}.
     */
    private void doAssertions(final FlowTreeNode rootFlowTreeNode) {

        assert rootFlowTreeNode != null;

        this.getProject().log("Running assertion on the following flow tree node: ", Project.MSG_DEBUG);
        this.getProject().log(rootFlowTreeNode.toString(), Project.MSG_DEBUG);

        for (final AssertFlowTreeNode assertFlowTreeNode : this.assertionsOnFlowTreeNode) {
            final String path = assertFlowTreeNode.getPath();
            if (path == null) {
                throw new BuildAssertParamMissingException("path", this.getLocation());
            }

            if (path.isEmpty()) {
                this.getProject().log("Asserting the root flow tree node.", Project.MSG_VERBOSE);
            } else {
                this.getProject().log(String.format("Asserting flow tree node identified by the path '%s'.", path),
                        Project.MSG_VERBOSE);
            }

            try {
                final FlowTreeNode selectedFlowTreeNode = this.selectFlowTreeNode(rootFlowTreeNode, path);

                for (final FlowTreeNodeAssertion assertion : assertFlowTreeNode.getAssertions()) {
                    assertion.doAssert(selectedFlowTreeNode, this.getProject(), this.getLocation());
                }
            } catch (final NumberFormatException e) {
                throw new BuildAssertParamInvalidValueException("path", path, this.getLocation(), e);
            } catch (final BuildAssertFlowTreeNodeNotFoundException e) {
                throw new BuildAssertFlowTreeNodeNotFoundException(path, this.getLocation(), e);
            }

        }
    }

    /**
     * Select a flow tree node from a root flow tree node and a path expression.
     * 
     * @param rootFlowTreeNode
     *            A root flow tree node from which the flow tree node given by {@code path} will be retrieved. Not
     *            {@code null}.
     * @param path
     *            A path expression identifying a flow tree node from the given root {@code rootFlowTreeNode}. Not
     *            {@code null}. Can be empty.
     * @return The flow tree node identified by the given root tree node and and the expression {@code path}.
     * @throws NumberFormatException
     *             The path expression is invalid.
     * @throws BuildAssertFlowTreeNodeNotFoundException
     *             No flow tree node found for the given {@code path}.
     */
    private FlowTreeNode selectFlowTreeNode(final FlowTreeNode rootFlowTreeNode, final String path)
            throws NumberFormatException, BuildAssertFlowTreeNodeNotFoundException {

        assert rootFlowTreeNode != null;
        assert path != null;

        final StringTokenizer tokenizer = new StringTokenizer(path, ".");
        while (tokenizer.hasMoreTokens()) {
            final String childIdStr = tokenizer.nextToken();
            final int childId = Integer.parseInt(childIdStr);
            if (rootFlowTreeNode.getFlowTreeNodes().size() < childId) {
                throw new BuildAssertFlowTreeNodeNotFoundException(this.getLocation());
            } else {
                return this.selectFlowTreeNode(rootFlowTreeNode.getFlowTreeNodes().get(childId),
                        path.indexOf('.') == -1 ? "" : path.substring(path.indexOf('.') + 1));
            }
        }
        return rootFlowTreeNode;

    }

}
