/**
 * Copyright (c) 2015-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.ant.task;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.io.StringReader;

import javax.xml.parsers.DocumentBuilder;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.util.ReaderInputStream;
import org.junit.Rule;
import org.junit.Test;
import org.ow2.petals.jmx.api.api.ComponentClient;
import org.ow2.petals.jmx.api.api.exception.AdminServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.ComponentErrorException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.NotRemoteJMXClientException;
import org.ow2.petals.jmx.api.mock.junit.PetalsJmxApiJunitRule;
import org.ow2.petals.jmx.api.mock.junit.PetalsJmxApiJunitRule.ComponentType;
import org.ow2.petals.jmx.api.mock.junit.SharedLibrary;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;

public class ListBindingComponentsTaskTest {

    @Rule
    public final PetalsJmxApiJunitRule jmxApi = new PetalsJmxApiJunitRule();

    /**
     * <p>
     * Check the Ant task 'jbi-list-binding-components' to get binding component through several sub-tests:
     * <ol>
     * <li>a shared library name is given without the version,</li>
     * <li>a shared library version is given without the name,</li>
     * <li>a shared library is correctly set as filtering criteria</li>
     * </ol>
     * </p>
     * <p>
     * Environment:
     * <ul>
     * <li>2 SLs installed: SL1, SL2,</li>
     * <li>several components installed:
     * <ul>
     * <li>a BC, started, using both SL1 and SL2,</li>
     * <li>a BC, started, using no SL,</li>
     * <li>a SE, started, using both SL1 and SL2,</li>
     * <li>a SE, started, using no SL,</li>
     * </ul>
     * </li>
     * </ul>
     * </p>
     * <p>
     * Expected result:
     * <ol>
     * <li>an error occurs about the missing version of the shared library,</li>
     * <li>an error occurs about the missing name of the shared library,</li>
     * <li>no error occurs, only binding components requiring the given shared library are returned.</li>
     * </ol>
     * </p>
     */
    @Test
    public void getComponentBySharedLibrary() throws AdminServiceErrorException, ComponentErrorException,
            InstallationServiceErrorException, SAXException, IOException, NotRemoteJMXClientException {

        final String slName1 = "sl-used-1";
        final String slVersion1 = "version-1";
        final String slName2 = "sl-used-2";
        final String slVersion2 = "version-2";
        final String bcUsingSlName = "binding-1";
        final String bcNotUsingSlName = "binding-2";
        final String seUsingSlName = "engine-1";
        final String seNotUsingSlName = "engine-2";

        // Prepare the JBI environment mocked
        
        // SL installed
        final SharedLibrary sl1 = new SharedLibrary(slName1, slVersion1);
        this.jmxApi.addSharedLibrary(sl1);
        final SharedLibrary sl2 = new SharedLibrary(slName2, slVersion2);
        this.jmxApi.addSharedLibrary(sl2);
        
        // component STARTED
        this.jmxApi.addComponentClient(bcUsingSlName, ComponentType.BINDING, new SharedLibrary[] { sl1, sl2 });
        this.jmxApi.addComponentClient(bcNotUsingSlName, ComponentType.BINDING);
        this.jmxApi.addComponentClient(seUsingSlName, ComponentType.ENGINE, new SharedLibrary[] { sl1, sl2 });
        this.jmxApi.addComponentClient(seNotUsingSlName, ComponentType.ENGINE);

        // Test #1 - A shared library name is given without the version
        {
            final ListBindingComponentsTask lbct = new ListBindingComponentsTask();
            lbct.setPort(String.valueOf(this.jmxApi.getPort()));
            lbct.setUsername(this.jmxApi.getUsername());
            lbct.setPassword(this.jmxApi.getPassword());
            lbct.setSharedLibraryName(slName1);
            lbct.setProject(new Project());

            try {
                lbct.execute();
                fail("No error occurs about the missing SL version");
            } catch (final BuildException e) {
                // NOP: Expected exception
            }
        }

        // Test #2 - A shared library version is given without the name
        {
            final ListBindingComponentsTask lbct = new ListBindingComponentsTask();
            lbct.setPort(String.valueOf(this.jmxApi.getPort()));
            lbct.setUsername(this.jmxApi.getUsername());
            lbct.setPassword(this.jmxApi.getPassword());
            lbct.setSharedLibraryVersion(slVersion1);
            lbct.setProject(new Project());

            try {
                lbct.execute();
                fail("No error occurs about the missing SL name");
            } catch (final BuildException e) {
                // NOP: Expected exception
            }
        }

        // Test #3 - A shared library is correctly set, with name and version
        {
            // Prepare ANt task
            final String resultProperty = "result-property";
            final ListBindingComponentsTask lbct = new ListBindingComponentsTask();
            lbct.setPort(String.valueOf(this.jmxApi.getPort()));
            lbct.setUsername(this.jmxApi.getUsername());
            lbct.setPassword(this.jmxApi.getPassword());
            lbct.setSharedLibraryName(slName1);
            lbct.setSharedLibraryVersion(slVersion1);
            lbct.setXmlOutput(resultProperty);
            lbct.setProject(new Project());

            // Execute Ant task
            lbct.execute();
            final String result = lbct.getProject().getProperty(resultProperty);

            // Assert results
            final DocumentBuilder docBuilder = DocumentBuilders.takeDocumentBuilder();
            final Reader resultReader = new StringReader(result);
            final InputStream resultInputStream = new ReaderInputStream(resultReader);
            final Document resultDoc = docBuilder.parse(resultInputStream);
            final Element componentInfoListElement = resultDoc.getDocumentElement();
            assertEquals("http://java.sun.com/xml/ns/jbi/component-info-list",
                    componentInfoListElement.getNamespaceURI());
            assertEquals("component-info-list", componentInfoListElement.getNodeName());
            assertEquals("1.0", componentInfoListElement.getAttribute("version"));
            final NodeList componentInfos = componentInfoListElement.getElementsByTagNameNS(
                    "http://java.sun.com/xml/ns/jbi/component-info-list", "component-info");
            assertNotNull(componentInfos);
            assertEquals(1, componentInfos.getLength());
            final Node componentInfo = componentInfos.item(0);
            assertTrue(componentInfo instanceof Element);
            final Element componentInfoElt = (Element) componentInfo;
            assertEquals("binding-component", componentInfoElt.getAttribute("type"));
            assertEquals(bcUsingSlName, componentInfoElt.getAttribute("name"));
            assertEquals(ComponentClient.State.STARTED.toString(), componentInfoElt.getAttribute("state"));
        }
    }

}
