/**
 * Copyright (c) 2015-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.ant.task;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;

import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.io.StringReader;

import javax.xml.parsers.DocumentBuilder;

import org.apache.tools.ant.Project;
import org.apache.tools.ant.util.ReaderInputStream;
import org.junit.Rule;
import org.junit.Test;
import org.ow2.petals.jmx.api.api.exception.AdminServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.ComponentErrorException;
import org.ow2.petals.jmx.api.api.exception.InstallationServiceErrorException;
import org.ow2.petals.jmx.api.api.exception.NotRemoteJMXClientException;
import org.ow2.petals.jmx.api.mock.junit.PetalsJmxApiJunitRule;
import org.ow2.petals.jmx.api.mock.junit.SharedLibrary;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;

import com.ebmwebsourcing.easycommons.xml.DocumentBuilders;

public class ListSharedLibrariesTaskTest {

    @Rule
    public final PetalsJmxApiJunitRule jmxApi = new PetalsJmxApiJunitRule();

    /**
     * <p>
     * Check the Ant task 'jbi-list-shared-libraries' to get shared libraries through several sub-tests:
     * <ol>
     * <li>a shared library name is given without the version,</li>
     * <li>a shared library version is given without the name,</li>
     * <li>a shared library is correctly set as filtering criteria</li>
     * </ol>
     * </p>
     * <p>
     * Environment:
     * <ul>
     * <li>3 SLs installed: SL1, SL2, SL3. SL1 and SL2 have the same version</li>
     * </ul>
     * </p>
     * <p>
     * Expected result:
     * <ol>
     * <li>no error occurs, only shared libraries with the given name are returned,</li>
     * <li>no error occurs, only shared libraries with the given version are returned,</li>
     * <li>no error occurs, only binding components requiring the given shared library are returned.</li>
     * </ol>
     * </p>
     */
    @Test
    public void getSharedLibrary() throws AdminServiceErrorException, ComponentErrorException,
            InstallationServiceErrorException, SAXException, IOException, NotRemoteJMXClientException {

        final String slName1 = "sl-used-1";
        final String slVersion1 = "version-1";
        final String slName2 = "sl-used-2";
        final String slVersion2 = "version-2";
        final String resultProperty = "result-property";

        // Prepare the JBI environment mocked
        
        // SL installed
        final SharedLibrary sl1 = new SharedLibrary(slName1, slVersion1);
        this.jmxApi.addSharedLibrary(sl1);
        final SharedLibrary sl2 = new SharedLibrary(slName2, slVersion1);
        this.jmxApi.addSharedLibrary(sl2);
        final SharedLibrary sl3 = new SharedLibrary(slName2, slVersion2);
        this.jmxApi.addSharedLibrary(sl3);

        // Test #1 - A shared library name is given without the version
        {
            final ListSharedLibrariesTask lset = new ListSharedLibrariesTask();
            lset.setPort(String.valueOf(this.jmxApi.getPort()));
            lset.setUsername(this.jmxApi.getUsername());
            lset.setPassword(this.jmxApi.getPassword());
            lset.setSharedLibraryName(slName1);
            lset.setXmlOutput(resultProperty);
            lset.setProject(new Project());

            lset.execute();
            final String result = lset.getProject().getProperty(resultProperty);

            // Assert results
            final DocumentBuilder docBuilder = DocumentBuilders.takeDocumentBuilder();
            final Reader resultReader = new StringReader(result);
            final InputStream resultInputStream = new ReaderInputStream(resultReader);
            final Document resultDoc = docBuilder.parse(resultInputStream);
            final Element sharedLibraryInfoListElement = resultDoc.getDocumentElement();
            assertEquals("http://java.sun.com/xml/ns/jbi/shared-library-info-list",
                    sharedLibraryInfoListElement.getNamespaceURI());
            assertEquals("shared-library-info-list", sharedLibraryInfoListElement.getNodeName());
            assertEquals("1.0", sharedLibraryInfoListElement.getAttribute("version"));
            final NodeList sharedLibraryInfos = sharedLibraryInfoListElement.getElementsByTagNameNS(
                    "http://java.sun.com/xml/ns/jbi/shared-library-info-list", "shared-library-info");
            assertNotNull(sharedLibraryInfos);
            assertEquals(1, sharedLibraryInfos.getLength());
            final Node sharedLibraryInfo = sharedLibraryInfos.item(0);
            assertTrue(sharedLibraryInfo instanceof Element);
            final Element sharedLibraryInfoElt = (Element) sharedLibraryInfo;
            assertEquals(slName1, sharedLibraryInfoElt.getAttribute("name"));
            assertEquals(slVersion1, sharedLibraryInfoElt.getAttribute("version"));
        }

        // Test #2 - A shared library version is given without the name
        {
            final ListSharedLibrariesTask lset = new ListSharedLibrariesTask();
            lset.setPort(String.valueOf(this.jmxApi.getPort()));
            lset.setUsername(this.jmxApi.getUsername());
            lset.setPassword(this.jmxApi.getPassword());
            lset.setSharedLibraryVersion(slVersion1);
            lset.setXmlOutput(resultProperty);
            lset.setProject(new Project());

            lset.execute();
            final String result = lset.getProject().getProperty(resultProperty);

            // Assert results
            final DocumentBuilder docBuilder = DocumentBuilders.takeDocumentBuilder();
            final Reader resultReader = new StringReader(result);
            final InputStream resultInputStream = new ReaderInputStream(resultReader);
            final Document resultDoc = docBuilder.parse(resultInputStream);
            final Element sharedLibraryInfoListElement = resultDoc.getDocumentElement();
            assertEquals("http://java.sun.com/xml/ns/jbi/shared-library-info-list",
                    sharedLibraryInfoListElement.getNamespaceURI());
            assertEquals("shared-library-info-list", sharedLibraryInfoListElement.getNodeName());
            assertEquals("1.0", sharedLibraryInfoListElement.getAttribute("version"));
            final NodeList sharedLibraryInfos = sharedLibraryInfoListElement.getElementsByTagNameNS(
                    "http://java.sun.com/xml/ns/jbi/shared-library-info-list", "shared-library-info");
            assertNotNull(sharedLibraryInfos);
            assertEquals(2, sharedLibraryInfos.getLength());
        }

        // Test #3 - A shared library is set with name and version
        {
            // Prepare ANt task
            final ListSharedLibrariesTask lset = new ListSharedLibrariesTask();
            lset.setPort(String.valueOf(this.jmxApi.getPort()));
            lset.setUsername(this.jmxApi.getUsername());
            lset.setPassword(this.jmxApi.getPassword());
            lset.setSharedLibraryName(slName2);
            lset.setSharedLibraryVersion(slVersion2);
            lset.setXmlOutput(resultProperty);
            lset.setProject(new Project());

            // Execute Ant task
            lset.execute();
            final String result = lset.getProject().getProperty(resultProperty);

            // Assert results
            final DocumentBuilder docBuilder = DocumentBuilders.takeDocumentBuilder();
            final Reader resultReader = new StringReader(result);
            final InputStream resultInputStream = new ReaderInputStream(resultReader);
            final Document resultDoc = docBuilder.parse(resultInputStream);
            final Element sharedLibraryInfoListElement = resultDoc.getDocumentElement();
            assertEquals("http://java.sun.com/xml/ns/jbi/shared-library-info-list",
                    sharedLibraryInfoListElement.getNamespaceURI());
            assertEquals("shared-library-info-list", sharedLibraryInfoListElement.getNodeName());
            assertEquals("1.0", sharedLibraryInfoListElement.getAttribute("version"));
            final NodeList sharedLibraryInfos = sharedLibraryInfoListElement.getElementsByTagNameNS(
                    "http://java.sun.com/xml/ns/jbi/shared-library-info-list", "shared-library-info");
            assertNotNull(sharedLibraryInfos);
            assertEquals(1, sharedLibraryInfos.getLength());
            final Node sharedLibraryInfo = sharedLibraryInfos.item(0);
            assertTrue(sharedLibraryInfo instanceof Element);
            final Element sharedLibraryInfoElt = (Element) sharedLibraryInfo;
            assertEquals(slName2, sharedLibraryInfoElt.getAttribute("name"));
            assertEquals(slVersion2, sharedLibraryInfoElt.getAttribute("version"));
        }
    }

}
