/**
 * Copyright (c) 2015-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.ant.task.monit;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;

import java.net.URISyntaxException;
import java.util.Set;

import org.apache.tools.ant.Project;
import org.junit.Test;
import org.ow2.petals.ant.task.monit.exception.BuildCriteriaSelectionTooLargeException;
import org.ow2.petals.ant.task.monit.exception.BuildExclusifParametersException;
import org.ow2.petals.ant.task.monit.exception.BuildNoFlowStepMatchException;
import org.ow2.petals.ant.task.monit.exception.BuildPropertyMissingException;
import org.ow2.petals.ant.task.monit.exception.BuildRefIdEmptyException;
import org.ow2.petals.ant.task.monit.exception.BuildRefIdInvalidException;
import org.ow2.petals.ant.task.monit.exception.BuildRefIdMissingException;
import org.ow2.petals.ant.task.monit.exception.BuildRowInvalidException;
import org.ow2.petals.commons.log.FlowLogData;
import org.ow2.petals.commons.log.TraceCode;
import org.ow2.petals.log.parser.api.model.Flow;

public class ExtractFlowStepInfoTaskTest extends AbstractMonitTraceTest {

    /**
     * Check the flow step information extraction from MONIT traces on nominal conditions when criterias (without row
     * number and not randmoly) select one flow step.
     */
    @Test
    public void nominal() throws URISyntaxException {

        final Project project = this.loadMonitTraces();

        final FlowStepCriteria traceCodeCrit = new FlowStepCriteria();
        traceCodeCrit.setName(FlowLogData.FLOW_STEP_TRACE_CODE);
        traceCodeCrit.setValue(TraceCode.CONSUME_EXT_FLOW_STEP_BEGIN.toString());

        final FlowStepCriteria flowStepIdCrit = new FlowStepCriteria();
        flowStepIdCrit.setName(FlowLogData.FLOW_STEP_ID_PROPERTY_NAME);
        flowStepIdCrit.setValue("5534db51-1c10-11e5-8de7-56847afe9799");

        final String propertyName = FlowLogData.FLOW_INSTANCE_ID_PROPERTY_NAME;
        final FlowStepInfo flowStepInfo = new FlowStepInfo();
        flowStepInfo.setProperty(propertyName);
        flowStepInfo.setAttribute("flowInstanceId");

        final ExtractFlowStepInfoTask efsiTask = new ExtractFlowStepInfoTask();
        efsiTask.setRefid(REF_ID);
        efsiTask.addFlowStepCriteria(traceCodeCrit);
        efsiTask.addFlowStepCriteria(flowStepIdCrit);
        efsiTask.addFlowStepInfo(flowStepInfo);
        efsiTask.setProject(project);

        efsiTask.execute();

        assertEquals("5534db50-1c10-11e5-8de7-56847afe9799", project.getProperty(propertyName));
    }

    /**
     * Check the flow step information extraction from MONIT traces on nominal conditions with a row number set.
     */
    @Test
    public void nominalWithRowGiven() throws URISyntaxException {

        final Project project = this.loadMonitTraces();

        final FlowStepCriteria traceCodeCrit = new FlowStepCriteria();
        traceCodeCrit.setName(FlowLogData.FLOW_STEP_TRACE_CODE);
        traceCodeCrit.setValue(TraceCode.CONSUME_EXT_FLOW_STEP_BEGIN.toString());

        final FlowStepCriteria flowStepInterfaceNameCrit = new FlowStepCriteria();
        flowStepInterfaceNameCrit.setName(FlowLogData.FLOW_STEP_INTERFACE_NAME_PROPERTY_NAME);
        flowStepInterfaceNameCrit.setValue("{http://petals.ow2.org/samples/se-bpmn/vacationService}vacation");

        final String propertyName = FlowLogData.FLOW_INSTANCE_ID_PROPERTY_NAME;
        final FlowStepInfo flowStepInfo = new FlowStepInfo();
        flowStepInfo.setProperty(propertyName);
        flowStepInfo.setAttribute("flowInstanceId");

        final ExtractFlowStepInfoTask efsiTask = new ExtractFlowStepInfoTask();
        efsiTask.setRefid(REF_ID);
        efsiTask.addFlowStepCriteria(traceCodeCrit);
        efsiTask.addFlowStepCriteria(flowStepInterfaceNameCrit);
        efsiTask.setRow(0);
        efsiTask.addFlowStepInfo(flowStepInfo);
        efsiTask.setProject(project);

        efsiTask.execute();

        assertEquals("55178f50-1c10-11e5-8de7-56847afe9799", project.getProperty(propertyName));
    }

    /**
     * Check the flow step information extraction from MONIT traces on nominal conditions with random selection.
     */
    @Test
    public void nominalRandomly() throws URISyntaxException {

        final Project project = this.loadMonitTraces();

        final FlowStepCriteria traceCodeCrit = new FlowStepCriteria();
        traceCodeCrit.setName(FlowLogData.FLOW_STEP_TRACE_CODE);
        traceCodeCrit.setValue(TraceCode.CONSUME_EXT_FLOW_STEP_BEGIN.toString());

        final FlowStepCriteria flowStepInterfaceNameCrit = new FlowStepCriteria();
        flowStepInterfaceNameCrit.setName(FlowLogData.FLOW_STEP_INTERFACE_NAME_PROPERTY_NAME);
        flowStepInterfaceNameCrit.setValue("{http://petals.ow2.org/samples/se-bpmn/vacationService}vacation");

        final String propertyName = FlowLogData.FLOW_INSTANCE_ID_PROPERTY_NAME;
        final FlowStepInfo flowStepInfo = new FlowStepInfo();
        flowStepInfo.setProperty(propertyName);
        flowStepInfo.setAttribute("flowInstanceId");

        final ExtractFlowStepInfoTask efsiTask = new ExtractFlowStepInfoTask();
        efsiTask.setRefid(REF_ID);
        efsiTask.addFlowStepCriteria(traceCodeCrit);
        efsiTask.addFlowStepCriteria(flowStepInterfaceNameCrit);
        efsiTask.setSelectRandomly(true);
        efsiTask.addFlowStepInfo(flowStepInfo);
        efsiTask.setProject(project);

        efsiTask.execute();

        final String actualFlowInstanceId = project.getProperty(propertyName);
        assertNotNull(actualFlowInstanceId);
        assertFalse(actualFlowInstanceId.isEmpty());
        boolean flowFound = false;
        final Set<Flow> flows = (Set<Flow>) project.getReference(REF_ID);
        for (final Flow flow : flows) {
            if (flow.getFlowId().equals(actualFlowInstanceId)) {
                flowFound = true;
            }
        }
        assertTrue(flowFound);
    }

    /**
     * Check the flow step information extraction from MONIT traces where:
     * <ul>
     * <li>only one flow step matches criterias,</li>
     * <li>the property name is not set.</li>
     * </ul>
     */
    @Test(expected = BuildPropertyMissingException.class)
    public void propertyNameNotSet() throws URISyntaxException {

        final Project project = this.loadMonitTraces();

        final FlowStepCriteria traceCodeCrit = new FlowStepCriteria();
        traceCodeCrit.setName(FlowLogData.FLOW_STEP_TRACE_CODE);
        traceCodeCrit.setValue(TraceCode.CONSUME_EXT_FLOW_STEP_BEGIN.toString());

        final FlowStepCriteria flowStepInterfaceNameCrit = new FlowStepCriteria();
        flowStepInterfaceNameCrit.setName(FlowLogData.FLOW_STEP_INTERFACE_NAME_PROPERTY_NAME);
        flowStepInterfaceNameCrit.setValue("{http://petals.ow2.org/samples/se-bpmn/vacationService}vacation");

        final String propertyName = FlowLogData.FLOW_INSTANCE_ID_PROPERTY_NAME;
        final FlowStepInfo flowStepInfo = new FlowStepInfo();
        flowStepInfo.setAttribute("flowInstanceId");
        assertNull(flowStepInfo.getProperty());

        final ExtractFlowStepInfoTask efsiTask = new ExtractFlowStepInfoTask();
        efsiTask.setRefid(REF_ID);
        efsiTask.addFlowStepCriteria(traceCodeCrit);
        efsiTask.addFlowStepCriteria(flowStepInterfaceNameCrit);
        efsiTask.setRow(0);
        efsiTask.addFlowStepInfo(flowStepInfo);
        efsiTask.setProject(project);

        efsiTask.execute();

        assertEquals("04717da0-1a84-11e5-84d5-56847afe9799", project.getProperty(propertyName));
    }

    /**
     * Check the flow step information extraction from MONIT traces when criterias are too large.
     */
    @Test(expected = BuildCriteriaSelectionTooLargeException.class)
    public void criteriasTooLarge() throws URISyntaxException {

        final Project project = this.loadMonitTraces();

        final FlowStepCriteria traceCodeCrit = new FlowStepCriteria();
        traceCodeCrit.setName(FlowLogData.FLOW_STEP_TRACE_CODE);
        traceCodeCrit.setValue(TraceCode.CONSUME_EXT_FLOW_STEP_BEGIN.toString());

        final FlowStepCriteria flowStepInterfaceNameCrit = new FlowStepCriteria();
        flowStepInterfaceNameCrit.setName(FlowLogData.FLOW_STEP_INTERFACE_NAME_PROPERTY_NAME);
        flowStepInterfaceNameCrit.setValue("{http://petals.ow2.org/samples/se-bpmn/vacationService}vacation");

        final String propertyName = FlowLogData.FLOW_INSTANCE_ID_PROPERTY_NAME;
        final FlowStepInfo flowStepInfo = new FlowStepInfo();
        flowStepInfo.setProperty(propertyName);
        flowStepInfo.setAttribute("flowInstanceId");

        final ExtractFlowStepInfoTask efsiTask = new ExtractFlowStepInfoTask();
        efsiTask.setRefid(REF_ID);
        efsiTask.addFlowStepCriteria(traceCodeCrit);
        efsiTask.addFlowStepCriteria(flowStepInterfaceNameCrit);
        efsiTask.addFlowStepInfo(flowStepInfo);
        efsiTask.setProject(project);

        efsiTask.execute();
    }

    /**
     * Check the flow step information extraction from MONIT traces when criterias are too large and a row number is set
     * to an unexisting row.
     */
    @Test(expected = BuildRowInvalidException.class)
    public void rowInvalidvalue() throws URISyntaxException {

        final Project project = this.loadMonitTraces();

        final FlowStepCriteria traceCodeCrit = new FlowStepCriteria();
        traceCodeCrit.setName(FlowLogData.FLOW_STEP_TRACE_CODE);
        traceCodeCrit.setValue(TraceCode.CONSUME_EXT_FLOW_STEP_BEGIN.toString());

        final FlowStepCriteria flowStepInterfaceNameCrit = new FlowStepCriteria();
        flowStepInterfaceNameCrit.setName(FlowLogData.FLOW_STEP_INTERFACE_NAME_PROPERTY_NAME);
        flowStepInterfaceNameCrit.setValue("{http://petals.ow2.org/samples/se-bpmn/vacationService}vacation");

        final String propertyName = FlowLogData.FLOW_INSTANCE_ID_PROPERTY_NAME;
        final FlowStepInfo flowStepInfo = new FlowStepInfo();
        flowStepInfo.setProperty(propertyName);
        flowStepInfo.setAttribute("flowInstanceId");

        final ExtractFlowStepInfoTask efsiTask = new ExtractFlowStepInfoTask();
        efsiTask.setRefid(REF_ID);
        efsiTask.addFlowStepCriteria(traceCodeCrit);
        efsiTask.addFlowStepCriteria(flowStepInterfaceNameCrit);
        efsiTask.setRow(5);
        efsiTask.addFlowStepInfo(flowStepInfo);
        efsiTask.setProject(project);

        efsiTask.execute();
    }

    /**
     * Check the processing of Ant task {@link ReadLogFilesTask} when the parameter 'refid' is missing.
     */
    @Test(expected = BuildRefIdMissingException.class)
    public void noId() throws URISyntaxException {

        final ExtractFlowStepInfoTask efsiTask = new ExtractFlowStepInfoTask();

        efsiTask.execute();
    }

    /**
     * Check the processing of Ant task {@link ReadLogFilesTask} when the parameter 'refid' is empty.
     */
    @Test(expected = BuildRefIdEmptyException.class)
    public void emptyId() throws URISyntaxException {

        final ExtractFlowStepInfoTask efsiTask = new ExtractFlowStepInfoTask();
        efsiTask.setRefid("");

        efsiTask.execute();
    }

    /**
     * Check the flow step information extraction from MONIT traces when the reference object containing MONIT traces is
     * unknown (not previously loaded).
     */
    @Test(expected = BuildRefIdInvalidException.class)
    public void unexistingRefId() throws URISyntaxException {

        final ExtractFlowStepInfoTask efsiTask = new ExtractFlowStepInfoTask();
        efsiTask.setRefid(REF_ID);
        efsiTask.setProject(new Project());

        efsiTask.execute();
    }

    /**
     * Check the flow step information extraction from MONIT traces where no MONIT trace matches criterias.
     */
    @Test(expected = BuildNoFlowStepMatchException.class)
    public void noCriteriaMatches() throws URISyntaxException {

        final Project project = this.loadMonitTraces();

        final FlowStepCriteria aCrit = new FlowStepCriteria();
        aCrit.setName("a-criteria");
        aCrit.setValue("a-value");

        final ExtractFlowStepInfoTask efsiTask = new ExtractFlowStepInfoTask();
        efsiTask.setRefid(REF_ID);
        efsiTask.addFlowStepCriteria(aCrit);
        efsiTask.setProject(project);

        efsiTask.execute();
    }

    /**
     * Check the flow step information extraction from MONIT traces when criterias are too large and a row number is set
     * and random selection is enable.
     */
    @Test(expected = BuildExclusifParametersException.class)
    public void rowWithRandom() throws URISyntaxException {

        final Project project = this.loadMonitTraces();

        final ExtractFlowStepInfoTask efsiTask = new ExtractFlowStepInfoTask();
        efsiTask.setRefid(REF_ID);
        efsiTask.setRow(5);
        efsiTask.setSelectRandomly(true);
        efsiTask.setProject(project);

        efsiTask.execute();
    }

}
