/**
 * Copyright (c) 2015-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.ant.task.monit.assertion.flowstep;

import java.net.URISyntaxException;

import org.apache.tools.ant.Project;
import org.junit.Test;
import org.ow2.petals.ant.task.monit.assertion.flowinstance.AssertFlowInstanceTask;
import org.ow2.petals.ant.task.monit.assertion.flowstep.AssertFlowStep;
import org.ow2.petals.ant.task.monit.assertion.flowstep.AttributeEquals;
import org.ow2.petals.ant.task.monit.assertion.flowstep.exception.BuildAssertAttributeNameNotSetException;
import org.ow2.petals.ant.task.monit.assertion.flowstep.exception.BuildAssertAttributeNotFoundException;
import org.ow2.petals.ant.task.monit.assertion.flowstep.exception.BuildAssertEmptyAttributeValueException;
import org.ow2.petals.ant.task.monit.assertion.flowstep.exception.BuildAssertAttributeNotEqualsException;
import org.ow2.petals.commons.log.FlowLogData;
import org.ow2.petals.commons.log.TraceCode;

/**
 * Unit test of {@link AttributeEquals}, used through {@link AssertFlowInstanceTask}
 * 
 * @author Christophe DENEUX - Linagora
 *
 */
public class AttributeEqualsTest extends AbstractAssertAttributeTest {

    /**
     * Check the assertion against no attribute name given
     */
    @Test(expected = BuildAssertAttributeNameNotSetException.class)
    public void nameNotSet() throws URISyntaxException {

        final AttributeEquals attributeEquals = new AttributeEquals();
        attributeEquals.setValue("a-value");

        this.executeAssertion(attributeEquals);
    }

    /**
     * Check the assertion against an empty attribute name
     */
    @Test(expected = BuildAssertEmptyAttributeValueException.class)
    public void emptyName() throws URISyntaxException {

        final AttributeEquals attributeEquals = new AttributeEquals();
        attributeEquals.setName("");
    }

    /**
     * Check the assertion against an attribute name set with spaces
     */
    @Test(expected = BuildAssertEmptyAttributeValueException.class)
    public void spacedName() throws URISyntaxException {

        final AttributeEquals attributeEquals = new AttributeEquals();
        attributeEquals.setName("");
    }

    /**
     * Check the assertion against an empty value
     */
    @Test(expected = BuildAssertEmptyAttributeValueException.class)
    public void emptyValue() throws URISyntaxException {

        final AttributeEquals attributeEquals = new AttributeEquals();
        attributeEquals.setName("an-attribute-name");
        attributeEquals.setValue("");
    }

    /**
     * Check the assertion against a value set with spaces
     */
    @Test(expected = BuildAssertEmptyAttributeValueException.class)
    public void spacedValue() throws URISyntaxException {

        final AttributeEquals attributeEquals = new AttributeEquals();
        attributeEquals.setName("an-attribute-name");
        attributeEquals.setValue(" ");
    }

    /**
     * Check the assertion against an attribute name that does not exist on the flow step trace
     */
    @Test(expected = BuildAssertAttributeNotFoundException.class)
    public void attributeNameNotExistOnFlowStep() throws URISyntaxException {

        final AttributeEquals attributeEquals = new AttributeEquals();
        attributeEquals.setName("an-attribute-name");
        attributeEquals.setValue("a-value");

        this.executeAssertion(attributeEquals);
    }

    /**
     * Check the assertion when the actual value is not the expected value (ie. the assertion fails)
     */
    @Test(expected = BuildAssertAttributeNotEqualsException.class)
    public void fails() throws URISyntaxException {

        final AttributeEquals attributeEquals = new AttributeEquals();
        attributeEquals.setName(FlowLogData.FLOW_STEP_TRACE_CODE);
        // The actual value is 'consumeFlowStepBegin'
        attributeEquals.setValue(TraceCode.PROVIDE_FLOW_STEP_BEGIN.toString());

        this.executeAssertion(attributeEquals);
    }

    /**
     * Check the assertion on ending trace of the flow step
     */
    @Test
    public void onEnd() throws URISyntaxException {

        final AttributeEquals attributeEquals = new AttributeEquals();
        attributeEquals.setOnEnd(true);
        attributeEquals.setName(FlowLogData.FLOW_INSTANCE_ID_PROPERTY_NAME);
        attributeEquals.setValue(FLOW_INSTANCE_ID);

        this.executeAssertion(attributeEquals);
    }

    private final void executeAssertion(final AttributeEquals attributeEquals) throws URISyntaxException {
        final Project project = this.loadMonitTraces();

        final AssertFlowStep assertFlowStep = new AssertFlowStep();
        assertFlowStep.setFlowStepIdx(0);
        assertFlowStep.addAttributeEquals(attributeEquals);

        final AssertFlowInstanceTask afitTask_0 = new AssertFlowInstanceTask();
        afitTask_0.setRefid(REF_ID);
        afitTask_0.setFlowInstanceId(FLOW_INSTANCE_ID);
        afitTask_0.addAssertFlowStep(assertFlowStep);
        afitTask_0.setProject(project);

        afitTask_0.execute();
    }
}
