/**
 * Copyright (c) 2015-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.ant.task.monit.assertion.flowstep;

import java.net.URISyntaxException;
import java.util.HashMap;

import org.apache.tools.ant.Location;
import org.apache.tools.ant.Project;
import org.junit.Test;
import org.ow2.petals.ant.task.monit.assertion.flowinstance.AssertFlowInstanceTask;
import org.ow2.petals.ant.task.monit.assertion.flowstep.exception.BuildAssertAttributeNameNotSetException;
import org.ow2.petals.ant.task.monit.assertion.flowstep.exception.BuildAssertAttributeNotSetException;
import org.ow2.petals.ant.task.monit.assertion.flowstep.exception.BuildAssertEmptyAttributeValueException;
import org.ow2.petals.commons.log.FlowLogData;
import org.ow2.petals.log.parser.api.model.FlowStep;

/**
 * Unit test of {@link AttributeSet}, used through {@link AssertFlowInstanceTask}
 * 
 * @author Christophe DENEUX - Linagora
 *
 */
public class AttributeSetTest extends AbstractAssertAttributeTest {

    private static final String FLOW_INSTANCE_ID = "5534db50-1c10-11e5-8de7-56847afe9799";

    /**
     * Check the assertion against no attribute name given
     */
    @Test(expected = BuildAssertAttributeNameNotSetException.class)
    public void nameNotSet() throws URISyntaxException {

        final AttributeSet attributeSet = new AttributeSet();

        this.executeAssertion(attributeSet);
    }

    /**
     * Check the assertion against an empty attribute name
     */
    @Test(expected = BuildAssertEmptyAttributeValueException.class)
    public void emptyName() throws URISyntaxException {

        final AttributeSet attributeSet = new AttributeSet();
        attributeSet.setName("");
    }

    /**
     * Check the assertion against an attribute name set with spaces
     */
    @Test(expected = BuildAssertEmptyAttributeValueException.class)
    public void spacedName() throws URISyntaxException {

        final AttributeSet attributeSet = new AttributeSet();
        attributeSet.setName("  ");
    }

    /**
     * Check the assertion against an attribute unexisting in the flow step
     */
    @Test(expected = BuildAssertAttributeNotSetException.class)
    public void attributeUnexisting() throws URISyntaxException {

        final AttributeSet attributeSet = new AttributeSet();
        final String attrName = "attrName";
        attributeSet.setName(attrName);
        final FlowStep flowStep = new FlowStep("step-id");
        attributeSet.doAssert(flowStep, new Project(), new Location("my-file"));
    }

    /**
     * Check the assertion against an attribute having an empty value in the flow step
     */
    @Test(expected = BuildAssertAttributeNotSetException.class)
    public void attributeHavingEmptyValue() throws URISyntaxException {

        final AttributeSet attributeSet = new AttributeSet();
        final String attrName = "attrName";
        attributeSet.setName(attrName);
        final FlowStep flowStep = new FlowStep("step-id");
        flowStep.setBeginProperties(new HashMap<String, String>());
        flowStep.getBeginProperties().put(attrName, "");
        attributeSet.doAssert(flowStep, new Project(), new Location("my-file"));
    }

    /**
     * Check the assertion against an attribute having a spaced value in the flow step
     */
    @Test(expected = BuildAssertAttributeNotSetException.class)
    public void attributeHavingSpacedalue() throws URISyntaxException {

        final AttributeSet attributeSet = new AttributeSet();
        final String attrName = "attrName";
        attributeSet.setName(attrName);
        final FlowStep flowStep = new FlowStep("step-id");
        flowStep.setBeginProperties(new HashMap<String, String>());
        flowStep.getBeginProperties().put(attrName, "  ");
        attributeSet.doAssert(flowStep, new Project(), new Location("my-file"));
    }

    /**
     * Check the assertion on ending trace of the flow step
     */
    @Test
    public void onEnd() throws URISyntaxException {

        final AttributeSet attributeSet = new AttributeSet();
        attributeSet.setOnEnd(true);
        attributeSet.setName(FlowLogData.FLOW_INSTANCE_ID_PROPERTY_NAME);

        this.executeAssertion(attributeSet);
    }

    private final void executeAssertion(final AttributeSet attributeSet) throws URISyntaxException {
        final Project project = this.loadMonitTraces();

        final AssertFlowStep assertFlowStep = new AssertFlowStep();
        assertFlowStep.setFlowStepIdx(0);
        assertFlowStep.addAttributeSet(attributeSet);

        final AssertFlowInstanceTask afitTask_0 = new AssertFlowInstanceTask();
        afitTask_0.setRefid(REF_ID);
        afitTask_0.setFlowInstanceId(FLOW_INSTANCE_ID);
        afitTask_0.addAssertFlowStep(assertFlowStep);
        afitTask_0.setProject(project);

        afitTask_0.execute();
    }
}
